package lexSimp;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

import ob.core.CentralFactory;
import ob.core.FeatureVector;
import ob.nlp.Word;
import ob.nlp.corpus.ContextVectors;
import ob.nlp.util.NlpUtils;
import ob.util.IOUtils;

/**
 * @author Or Biran
 *
 */
public class Simplifier {

	private static final double THRESHOLD = 0.01;

	private boolean _verboseOutput = false;
	private int _window = 10;
	
	private String _workingDir;
	
	public Simplifier(String workingDir) {
		_workingDir = workingDir;
	}

	public void simplify(String inputFile) {
		String rulesFile = new File(_workingDir, SimplifierCreator.getRulesFileName(getFeaturesName())).getAbsolutePath();
		String vectorsFile = new File(_workingDir, SimplifierCreator.getRulesVectorsFileName(getFeaturesName())).getAbsolutePath();

		Map<Word, SortedMap<Double, Word>> rules = IOUtils.readRules(rulesFile);
		ContextVectors vectors = IOUtils.readContextVectors(vectorsFile);
		
		List<String> lines = Utils.readLines(new File(inputFile));
		
		for (String line : lines) {
			List<Integer> changed = new ArrayList<Integer>();
			List<String> words = NlpUtils.getWords(line);
			List<Word> contextWords = Utils.getContextWords(line);
			List<String> newWords = new ArrayList<String>();
			int index = 0;
			for (int i=0; i<words.size(); i++) {
				if (index < contextWords.size() - 1 && Utils.equalContextWord(contextWords.get(index).getText(), words.get(i))) {
					index++;
				}
				Word word = CentralFactory.getWord(words.get(i), null);
				if (rules.containsKey(word)) {
					Word replacement = word;
					FeatureVector wordVector = vectors.getVector(word);
					if (wordVector==null) continue;
					for (Word candidate : rules.get(word).values()) {
						FeatureVector candidateVector = vectors.getVector(candidate);
						if (candidateVector==null) continue;
						double similarity = Utils.similarityInContext(candidateVector, wordVector, contextWords, index, getFeaturesName());
						if (_verboseOutput)	{
							System.out.println(word + " -> " + candidate + ": " + similarity);
						}
						if (similarity > THRESHOLD) {
							replacement = candidate;
							changed.add(i);
						}
					}
					newWords.add(replacement.getText());
				}
				else newWords.add(word.getText());
			}

			String newText = "";
			if (_verboseOutput)	{
				System.out.println(line);
				newText += "-> ";
			}
			List<String> changedWords = new ArrayList<String>();
			for (int i=0; i<newWords.size(); i++) {
				String word = newWords.get(i);
				if (changed.contains(i) && _verboseOutput) {
					word = word.toUpperCase();
					changedWords.add(words.get(i) + " -> " + word);
				}
				newText += word + " ";
			}
			
			System.out.println(newText);
			
			if (_verboseOutput)	{
				System.out.println("changed: " + changedWords);
				System.out.println();
			}
		}
	}

	private String getFeaturesName() {
		return SimplifierCreator.getFeatureName(_window);
	}

	public void setWindow(int window) {
		_window = window;
	}

	public void setVerboseOutput(boolean verboseOutput) {
		_verboseOutput = verboseOutput;
	}

	
}
