package lexSimp.app;

import java.io.File;

import lexSimp.SimplifierCreator;

/**
 * @author Or Biran
 *
 */
public class SimplifierCreatorRunner {

	public static void main(String[] args) {
		if (args.length < 2 || args.length > 3) {
			System.out.println("USAGE: SimplifierCreatorRunner <corpusDir> <workingDir> [<window>]");
			System.out.println("<window> default - 10");
			System.exit(0);
		}

		String corpusDir = args[0]; //"data/corpus/wikipedia";
		String workingDir = args[1]; //"working/output/wikipedia2";
		Integer window = args.length == 3 ? Integer.parseInt(args[2]) : 10;

		new File(workingDir).mkdirs();
		
		SimplifierCreator creator = new SimplifierCreator(corpusDir, workingDir);
		creator.setWindow(window);

		// create and filter context vectors from the complex corpus
		creator.outputVectors();
		creator.consolidateVectors();
		creator.wordNetVectorFilter();

		// create and filter the similarity matrix from the context vectors
		creator.outputSimilarity();
		creator.wordNetSimilarityFilter();

		// create word frequencies for the simple and complex corpora
		creator.outputComplexWordFrequency();
		creator.outputSimpleWordFrequency();
		
		// create word complexity scores from the frequencies
		creator.outputComplexityScores();

		// create the simplification rules
		creator.outputRules();
		
		// create the full context vectors for words participating in the rules 
		creator.outputRulesVectors();
	}

	
	
}
