package ob.core;

import ob.nlp.POS;
import ob.nlp.Word;
import ob.nlp.WordImpl;
import ob.nlp.WordScores;
import ob.nlp.WordScoresImpl;
import ob.nlp.WordSimilarity;
import ob.nlp.WordSimilarityImpl;
import ob.nlp.corpus.ContextVectors;
import ob.nlp.corpus.ContextVectorsImpl;
import ob.nlp.morphology.Morphology;
import ob.nlp.morphology.morphAdorn.MorphAdornMorphology;


/**
 * @author Or Biran
 *
 * Provides methods returning implementations for various interfaces.  
 */
public class CentralFactory {

	public static WordScores getWordScores() {
		return getWordScores(0.0);
	}
	
	public static WordScores getWordScores(double initialScore) {
		WordScoresImpl scores = new WordScoresImpl();
		scores.setInitialScore(initialScore);
		return scores;
	}

	public static ContextVectors getContextVectors() {
		return new ContextVectorsImpl();
	}

	public static Word getWord(String text, POS pos) {
		return new WordImpl(text, pos);
	}

	public static FeatureVector getFeatureVector() {
		return new FeatureVectorImpl();
	}

	public static WordSimilarity getWordSimilarity() {
		return new WordSimilarityImpl();
	}

	public static Value getNumericValue(Double value) {
		return new NumericValue(value);
	}

	public static Value getStringValue(String value) {
		return new StringValue(value);
	}

	public static Morphology getMorphology() {
		return new MorphAdornMorphology();
	}

}
