package ob.core;

import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;



public class FeatureVectorImpl implements FeatureVector {

	private Map<Feature, Double> _map = new HashMap<Feature, Double>();
	
	private Double _initialScore = 0.0;
	
	private Double _euclideanNorm;



	@Override
	public int size() {
		return getFeatures().size();
	}

	@Override
	public void increment(Feature feature, double increment) {
		Double score = _map.get(feature);
		if (score==null) score = getInitialScore();
		_map.put(feature, score + increment);
		reset();
	}


	@Override
	public void consolidate(FeatureVector otherVector) {
		for (Feature feature : otherVector.getFeatures()) {
			increment(feature, otherVector.getValue(feature));
		}
		reset();
	}

	private void reset() {
		_euclideanNorm = null;
	}

	@Override
	public Collection<Feature> getFeatures() {
		return _map.keySet();
	}

	@Override
	public Double getValue(Feature feature) {
		return _map.get(feature);
	}


	public Double getInitialScore() {
		return _initialScore;
	}


	public void setInitialScore(Double initialScore) {
		_initialScore = initialScore;
	}


	@Override
	public void set(Feature feature, double value) {
		_map.put(feature, value);
	}

	@Override
	public double euclideanNorm() {
		if (_euclideanNorm == null) {
			double norm = 0.0;
			for (Feature feature : getFeatures()) {
				norm += Math.pow(getValue(feature), 2);
			}
			norm = Math.sqrt(norm);
			_euclideanNorm = norm;
		}
		return _euclideanNorm;
	}

	@Override
	public Collection<Feature> getSortedFeatures() {
		Collection<Feature> sortedFeatures = new TreeSet<Feature>(new Comparator<Feature>() {
			public int compare(Feature arg0, Feature arg1) {
				int valueComp = getValue(arg0).compareTo(getValue(arg1));
				if (valueComp != 0) return valueComp;
				return arg0.getName().compareTo(arg1.getName());
			}
		});
		sortedFeatures.addAll(getFeatures());
		return sortedFeatures;
	}

	@Override
	public void removeFeaturesBelow(double minScore) {
		for (Feature feature : _map.keySet()) {
			if (_map.get(feature) < minScore) _map.remove(feature);
		}
	}

}
