package ob.nlp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

public class MatrixWordSimilarity implements WordSimilarity {

	private static final int SIZE_UNIT = 1000;
	
	private int _sizeUnits = 1;

	private double[][] _matrix = newMatrix();
	private Map<Word, Integer> _words = new HashMap<Word, Integer>();
	
	
	@Override
	public Double get(Word word1, Word word2) {
		Integer index1 = _words.get(word1);
		if (index1 == null) return null;
		
		Integer index2 = _words.get(word2);
		if (index2 == null) return null;
		
		return _matrix[index1][index2];
	}

	@Override
	public Collection<Word> getFirstWords() {
		return _words.keySet();
	}

	@Override
	public Collection<Word> getSecondWords(Word firstWord) {
		Collection<Word> secondWords = new ArrayList<Word>();
		for (Word word : _words.keySet()) {
			if (word.equals(firstWord)) continue;
			secondWords.add(word);
		}
		return secondWords;
	}

	@Override
	public Collection<Similarity> getSimilarities() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void set(Word word1, Word word2, Double value) {
		if (! _words.containsKey(word1)) addWord(word1);
		if (! _words.containsKey(word2)) addWord(word2);
		_matrix[index(word1)][index(word2)] = value;
	}

	private int index(Word word) {
		return _words.get(word);
	}

	private void addWord(Word word) {
		int size = _words.size();
		
		if (matrixSize() == size) {
			increaseMatrixSize();
			double[][] newMatrix = newMatrix();
			for (Word oldWord1 : _words.keySet()) {
				for (Word oldWord2 : _words.keySet()) {
					newMatrix[index(oldWord1)][index(oldWord2)] = _matrix[index(oldWord1)][index(oldWord2)];
				}
			}
			
			_matrix = newMatrix;
		}
		
		_words.put(word, size);
	}

	private void increaseMatrixSize() {
		_sizeUnits++;
	}

	private double[][] newMatrix() {
		return new double[matrixSize()][matrixSize()];
	}

	private int matrixSize() {
		return SIZE_UNIT * _sizeUnits;
	}

	@Override
	public void consolidate(WordSimilarity otherSimilarity) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public int size() {
		// TODO Auto-generated method stub
		return 0;
	}

}
