package ob.nlp;

import java.util.ArrayList;
import java.util.List;

import ob.core.CentralFactory;
import ob.nlp.util.NlpUtils;


public class SimpleWordsToTerms implements WordsToTerms {

	@Override
	public List<Word> wordsToTerms(List<Word> words) {
		List<Word> terms = new ArrayList<Word>();
		
		for (Word word : words) {
			String text = word.getText();

			text = NlpUtils.removeGarbage(text);
			text = NlpUtils.removeEnclosures(text);
			text = text.replaceFirst("\\d+(th|st|nd|rd)", "#NTH#");
			text = text.replaceFirst("\\d+(\\d|,)+\\d+", "#N#");
			text = text.replaceFirst("\\d+", "#N#");
			
//			if (text.equals(word.getText()) == false) System.out.println(word.getText() + " => " + text);
			
//			if (isPunctuation(word)) continue;
			if (hasDigits(text)) continue;
			if (isPTBAnnotation(text)) continue;
			if (hasNoLetters(text)) continue;
			
			terms.add(CentralFactory.getWord(text, word.getPos()));
		}
		
		return terms;
	}

	private boolean hasDigits(String word) {
		for (char c : word.toCharArray()) {
			if (Character.isDigit(c)) return true;
		}
		return false;
	}

	private boolean hasNoLetters(String word) {
		for (char c : word.toCharArray()) {
			if (Character.isLetter(c)) return false;
		}
		return true;
	}

	private boolean isPTBAnnotation(String word) {
		if (word.equalsIgnoreCase("-RRB-")) return true;
		if (word.equalsIgnoreCase("-LRB-")) return true;
		if (word.equalsIgnoreCase("-RCB-")) return true;
		if (word.equalsIgnoreCase("-LCB-")) return true;
		if (word.equalsIgnoreCase("-RSB-")) return true;
		if (word.equalsIgnoreCase("-LSB-")) return true;
		return false;
	}

//	private boolean isNumber(Word word) {
//		boolean hasDigit = false;
//		for (char c : word.getText().toCharArray()) {
//			if (Character.isDigit(c)) hasDigit = true;
//			if (Character.isLetter(c)) return false;
//		}
//		return hasDigit;
//	}
//
//	private boolean isPunctuation(Word word) {
//		return English.isPunctuation(word.getText());
//	}
//
	
}
