package ob.nlp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import ob.core.CentralFactory;
import ob.util.Utils;


public class StopWordsFromFiles implements StopWords {

	private static final String DEFAULT_FILE = "data/stopWords/base_large.txt";
	
	private List<String> _files = new ArrayList<String>();

	private Collection<Word> _stopWords;
	
	public StopWordsFromFiles() {
		addDefaultFile();
	}
	
	public void addDefaultFile() {
		addFile(DEFAULT_FILE);
	}
	
	public void addFile(String filename) {
		_files.add(filename);
		_stopWords = null;
	}
	
	public void clearAllFiles() {
		_files = new ArrayList<String>();
		_stopWords = null;
	}
	
	@Override
	public boolean isStopWord(Word word) {
		return getStopWords().contains(word);
	}

	private Collection<Word> getStopWords() {
		if (_stopWords==null) {
			_stopWords = new HashSet<Word>();
			for (String file : _files) {
				_stopWords.addAll(readStopWordsFromFile(file));
			}
		}
		return _stopWords;
	}

	private Collection<Word> readStopWordsFromFile(String file) {
		List<String> lines = Utils.readLines(file);
		List<Word> words = new ArrayList<Word>();
		for (String line : lines) {
			words.add(CentralFactory.getWord(line, null));
		}
		return words;
	}
	
	
}
