package ob.nlp;

import java.util.Collection;
import java.util.HashSet;

import ob.core.CentralFactory;
import ob.core.Feature;
import ob.core.FeatureVector;


public class WordScoresImpl implements WordScores {

	private FeatureVector _vector = CentralFactory.getFeatureVector();
	private Collection<Word> _words;
	
	
	@Override
	public Double getScore(Word word) {
		return _vector.getValue(feature(word));
	}

	private Feature feature(Word word) {
		return new WordFeature(word.getText(), word);
	}
	
	private Word word(Feature feature) {
		return ((WordFeature)feature).getWord();
	}

	@Override
	public Collection<Word> getWords() {
		if (_words == null) {
			_words = new HashSet<Word>();
			for (Feature feature : _vector.getFeatures()) {
				_words.add(word(feature));
			}
		}
		return _words;
	}

	@Override
	public FeatureVector getVector() {
		return _vector;
	}

	@Override
	public void increment(Word word, Double increment) {
		_vector.increment(feature(word), increment);
		if (_words != null) _words.add(word);
	}

	@Override
	public void set(Word word, Double score) {
		_vector.set(feature(word), score);
		if (_words != null) _words.add(word);
	}

	public void setInitialScore(Double initialScore) {
		_vector.setInitialScore(initialScore);
	}

	@Override
	public int size() {
		return _vector.size();
	}

}
