package ob.nlp;

import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

public class WordSimilarityImpl implements WordSimilarity {

	private Collection<Similarity> _similarities = new TreeSet<Similarity>(new SimilarityComparator());
	private Map<Word, Map<Word, WordSimilarity.Similarity>> _map;
	
	@Override
	public Collection<Similarity> getSimilarities() {
		return _similarities;
	}

	@Override
	public void set(Word word1, Word word2, Double value) {
		WordSimilarity.Similarity similarity = new SimilarityImpl(word1, word2, value);
		_similarities.add(similarity);
		
		if (_map != null) addToMap(similarity);
	}

	@Override
	public Double get(Word word1, Word word2) {
		if (_map == null) buildMap();
		
		Map<Word, WordSimilarity.Similarity> secondMap = _map.get(word1);
		if (secondMap == null) return null;
		Similarity sim = secondMap.get(word2);
		return sim==null ? null : sim.getValue();
	}

	private void buildMap() {
		_map = new HashMap<Word, Map<Word, WordSimilarity.Similarity>>();
		
		for (WordSimilarity.Similarity similarity : _similarities) {
			addToMap(similarity);
		}
	}

	private void addToMap(Similarity similarity) {
		Map<Word, WordSimilarity.Similarity> secondMap = _map.get(similarity.getWord1());
		if (secondMap == null) secondMap = new HashMap<Word, WordSimilarity.Similarity>();
		secondMap.put(similarity.getWord2(), similarity);
		_map.put(similarity.getWord1(), secondMap);
	}

	@Override
	public Collection<Word> getFirstWords() {
		if (_map == null) buildMap();

		return _map.keySet();
	}

	@Override
	public Collection<Word> getSecondWords(Word firstWord) {
		if (_map == null) buildMap();

		Map<Word, WordSimilarity.Similarity> secondMap = _map.get(firstWord);
		if (secondMap == null) return null;
		return secondMap.keySet();
	}
	
	
	private class SimilarityComparator implements Comparator<WordSimilarity.Similarity> {
		
		public int compare(Similarity arg0, Similarity arg1) {
			if (arg0.equals(arg1)) return 0;
			int comp = arg1.getValue().compareTo(arg0.getValue());
			if (comp != 0) return comp;
			comp = arg0.getWord1().getText().compareTo(arg1.getWord1().getText());
			if (comp != 0) return comp;
			return arg0.getWord2().getText().compareTo(arg1.getWord2().getText());
		}
		
	}

	
	private class SimilarityImpl implements WordSimilarity.Similarity {
		
		private Word _word1;
		private Word _word2;
		private Double _value;
		
		public SimilarityImpl(Word word1, Word word2, Double value) {
			_word1 = word1;
			_word2 = word2;
			_value = value;
		}

		public Word getWord1() {return _word1;}
		public Word getWord2() {return _word2;}
		public Double getValue() {return _value;}

		@Override
		public boolean equals(Object obj) {
			if (obj == null) return false;
			if (obj == this) return true;
			if (obj instanceof Similarity) {
				Similarity si = (Similarity)obj;
				return _word1.equals(si.getWord1()) && _word2.equals(si.getWord2());
			}
			return false;
		}

		@Override
		public int hashCode() {
			int hash = 31;
			hash = (hash * 19) + _word1.hashCode();
			hash = (hash * 19) + _word2.hashCode();
			return hash;
		}		
	}


	@Override
	public void consolidate(WordSimilarity otherSimilarity) {
		for (Word word1 : otherSimilarity.getFirstWords()) {
			for (Word word2 : otherSimilarity.getSecondWords(word1)) {
				set(word1, word2, otherSimilarity.get(word1, word2));
			}
		}
	}

	@Override
	public int size() {
		return _similarities.size();
	}

}
