package ob.nlp.morphology.morphAdorn;

import java.util.ArrayList;
import java.util.Collection;

import edu.northwestern.at.utils.corpuslinguistics.inflector.DefaultInflector;
import edu.northwestern.at.utils.corpuslinguistics.inflector.Inflector;
import edu.northwestern.at.utils.corpuslinguistics.inflector.Person;
import edu.northwestern.at.utils.corpuslinguistics.inflector.VerbTense;
import edu.northwestern.at.utils.corpuslinguistics.lemmatizer.Lemmatizer;
import edu.northwestern.at.utils.corpuslinguistics.lemmatizer.LemmatizerFactory;
import ob.nlp.morphology.Morphology;

public class MorphAdornMorphology implements Morphology {

	private static final Collection<String> PERSONS = new ArrayList<String>();
	private static final Collection<String> VERB_TENSES = new ArrayList<String>();
	
	static {
		for (Person person : Person.values()) {
			PERSONS.add(person.name());
		}
		for (VerbTense tense : VerbTense.values()) {
			VERB_TENSES.add(tense.name());
		}
	}
	
	private Inflector _inflector = new DefaultInflector();
	private Lemmatizer _lemmatizer = new LemmatizerFactory().newLemmatizer();
	
	@Override
	public Collection<String> getPersons() {
		return PERSONS;
	}
	
	@Override
	public Collection<String> getVerbTenses() {
		return VERB_TENSES;
	}

	@Override
	public String conjugate(String lemma, String tense, String person) {
		return _inflector.conjugate(lemma, VerbTense.valueOf(tense), Person.valueOf(person));
	}

	@Override
	public String lemmatize(String word) {
		try {
			return _lemmatizer.lemmatize(word);
		} catch (NullPointerException e) {
			return null;
		}
	}
	
	@Override
	public String pluralize(String lemma, int number) {
		return _inflector.pluralize(lemma, number);
	}
	
}
