package ob.nlp.penn;

import ob.nlp.POS;
import ob.nlp.POSFactory;

/**
 * @author Or Biran
 *
 * Penn Treebank POS.
 */
public class PennPOS implements POS {

	public static final String PLURAL_SUFFIX = "S";
	public static final String VERB_PREFIX = "V";
	public static final String SIMPLE_NOUN_PREFIX = "NN";
	public static final String PROPER_NOUN_PREFIX = "NNP";
	public static final String NOUN_PREFIX = "N";

	public static final String PERSONAL_PRONOUN = "PRP";

	public static final String VERB_BASE_FORM = "VB";
	public static final String VERB_PAST_TENSE = "VBD";
	public static final String VERB_PAST_PARTICIPLE = "VBN";
	public static final String VERB_GERUND_OR_PRESENT_PARTICIPLE = "VBG";
	public static final String VERB_THIRD_PERSON_SINGULAR_PRESENT = "VBZ";
	public static final String VERB_NON_THIRD_PERSON_SINGULAR_PRESENT = "VBP";

	public static final String ADJECTIVE_PREFIX = "JJ";
	public static final String ADVERB_PREFIX = "RB";

	public static final String WH_DETERMINER = "WDT";
	public static final String WH_PRONOUN_POSSESSIVE = "WP$";
	public static final String WH_PRONOUN_OBJECTIVE_OR_NOMINATIVE = "WP";
	public static final String WH_PRONOUN_OBJECTIVE = "WPO";
	public static final String WH_PRONOUN_NOMINATIVE = "WPS";
	public static final String WH_PRONOUN_PREFIX = "WP";

	public static final String POSSESSIVE = "POS";
	public static final String PREPOSITION = "IN";
	
	public static final String INFINITE_MARKER_TO = "TO";

	public static final String CARDINAL_NUMERAL = "CD";

	public static final String PERIOD = ".";
	public static final String COMMA = ",";
	
	
	
	private String _string;


	public PennPOS(String string) {
		_string = string;
	}

	public String getString() {
		return _string;
	}

	public void setString(String string) {
		_string = string;
	}

	public boolean isVerb() {
		return getString().startsWith(VERB_PREFIX);
	}

	public boolean isSimpleNoun() {
		return getString().startsWith(SIMPLE_NOUN_PREFIX);
	}

	public boolean isPluralSimpleNoun() {
		return isSimpleNoun() && getString().endsWith(PLURAL_SUFFIX);
	}

	public boolean is(String string) {
		return getString().equals(string);
	}

	public boolean isVerbBaseForm() {
		return is(VERB_BASE_FORM);
	}

	public boolean isVerbPastTense() {
		return is(VERB_PAST_TENSE);
	}

	public boolean isVerbGerundOrPresentParticiple() {
		return is(VERB_GERUND_OR_PRESENT_PARTICIPLE);
	}

	public boolean isVerbPastParticiple() {
		return is(VERB_PAST_PARTICIPLE);
	}

	public boolean isVerbNonThirdPersonSingularPresent() {
		return is(VERB_NON_THIRD_PERSON_SINGULAR_PRESENT);
	}

	public boolean isVerbThirdPersonSingularPresent() {
		return is(VERB_THIRD_PERSON_SINGULAR_PRESENT);
	}

	public boolean isPossessive() {
		return is(POSSESSIVE);
	}

	public boolean isPeriod() {
		return is(PERIOD);
	}

	public boolean isComma() {
		return is(COMMA);
	}

	public boolean isWhDeterminer() {
		return is(WH_DETERMINER);
	}

	public boolean isWhDeterminerOrPronoun() {
		return isWhDeterminer() || isWhPronoun();
	}

	public boolean isWhPronoun() {
		return getString().startsWith(WH_PRONOUN_PREFIX);
	}

	public boolean isPreposition() {
		return is(PREPOSITION);
	}

	public boolean isNoun() {
		return isSimpleNoun() || is(PERSONAL_PRONOUN);
	}

	public boolean isWhPronounPossessive() {
		return is(WH_PRONOUN_POSSESSIVE);
	}

	public boolean isCardinalNumeral() {
		return is(CARDINAL_NUMERAL);
	}

	public boolean isInfiniteMarkerTo() {
		return is(INFINITE_MARKER_TO);
	}

	public static final POSFactory FACTORY = new PennPOSFactory();
	
	public POSFactory getFactory() {
		return FACTORY;
	}

	public boolean isPersonalPronoun() {
		return is(PERSONAL_PRONOUN);
	}

	public boolean isProperNoun() {
		return getString().startsWith(PROPER_NOUN_PREFIX);
	}

	public boolean isWhPronounObjective() {
		return is(WH_PRONOUN_OBJECTIVE);
	}

	public boolean isAdjective() {
		return getString().startsWith(ADJECTIVE_PREFIX);
	}

	public boolean isAdverb() {
		return getString().startsWith(ADVERB_PREFIX);
	}

	@Override
	public String getStem() {
		if (isVerb()) return VERB_BASE_FORM;
		if (isNoun()) return NOUN_PREFIX;
		if (isAdjective()) return ADJECTIVE_PREFIX;
		if (isAdverb()) return ADVERB_PREFIX;
		return getString();
	}
	
}
