package ob.nlp.util;

import java.util.ArrayList;
import java.util.List;

import edu.stanford.nlp.ling.Word;
import edu.stanford.nlp.process.DocumentPreprocessor;
import edu.stanford.nlp.process.WordToSentenceProcessor;

/**
 * @author Or Biran
 * 
 * Provides various NLP-related utility methods.
 */
public class NlpUtils {

	public static final String SPACE = " ";
	
	public static final String[] ENCLOSURES = {"(", ")", "{", "}", "[", "]", "\""};
	public static final String[] GARBAGE = {"\\", "/"};
	
	public static String preprocess(String text) {
		String cleanText = "";
		for (Word word : getStanfordWords(text)) {
			cleanText += word + SPACE;
		}
		cleanText = cleanText.trim();

		return cleanText;
	}
//	static SentenceSplitter _sentenceSplitter;
	
//	public static List<String> splitSentencesMorph(String text) {
//		if (_sentenceSplitter == null) {
//			_sentenceSplitter = new DefaultSentenceSplitter();
//
//			try {
//				PartOfSpeechGuesser partOfSpeechGuesser = new DefaultPartOfSpeechGuesser();
//				Lexicon lexicon = new DefaultWordLexicon();
//				partOfSpeechGuesser.setWordLexicon( lexicon );
//				Lexicon suffixLexicon = new DefaultSuffixLexicon();
//				partOfSpeechGuesser.setSuffixLexicon( suffixLexicon );
//				_sentenceSplitter.setPartOfSpeechGuesser( partOfSpeechGuesser );
//			} catch (Exception e) {
//				throw new RuntimeException(e);
//			}
//		}
//		
//		WordTokenizer wordTokenizer = new DefaultWordTokenizer();
//
//		List<List<String>> sentences = _sentenceSplitter.extractSentences(text, wordTokenizer);
//		
//		List<String> output = new ArrayList<String>();
//		for (List<String> sentence : sentences) {
//			String outputSentence = "";
//			for (String word : sentence) {
//				outputSentence += word + " ";
//			}
//			output.add(outputSentence.trim());
//		}
//		
//		return output;
//	}
//	
	public static List<String> splitSentences(String text) {
		List<Word> words = getStanfordWords(text);
		
		WordToSentenceProcessor<Word, String, String> processor = new WordToSentenceProcessor<Word, String, String>();
		List<List<Word>> sents = processor.process(words);

		List<String> sentences = new ArrayList<String>();

		for (List<Word> sent : sents) {
			String sentence = "";
			for (Word w : sent) {
				sentence += w + SPACE;
			}
			sentence = sentence.trim();
			sentences.add(sentence);
		}
		
		return sentences;
	}
	
	private static List<Word> getStanfordWords(String text) {
		DocumentPreprocessor preprocessor = new DocumentPreprocessor();
		return preprocessor.getWordsFromString(text);
	}

	public static List<String> getWords(String text) {
		List<String> stringWords = new ArrayList<String>();
		
		for (Word word : getStanfordWords(text)) {
			stringWords.add(word.word());
		}
		
		return stringWords;
	}
	
	public static String stem(String word) {
		return Porter.stripAffixes(word);
	}

	public static String normalizeLetters(String word) {
		String w = word;
		
		w = w.replace("", "a");
		w = w.replace("", "e");
		w = w.replace("", "i");
		w = w.replace("", "o");
		w = w.replace("", "u");
		
		w = w.replace("", "a");
		w = w.replace("", "e");
		w = w.replace("", "i");
		w = w.replace("", "o");
		w = w.replace("", "u");
		w = w.replace("", "y");
		
		w = w.replace("", "a");
		w = w.replace("", "e");
		w = w.replace("", "i");
		w = w.replace("", "o");
		w = w.replace("", "u");
		
		w = w.replace("", "a");
		w = w.replace("", "e");
		w = w.replace("", "i");
		w = w.replace("", "o");
		w = w.replace("", "u");

		w = w.replace("", "a");
		w = w.replace("", "o");
		w = w.replace("", "n");

		w = w.replace("", "a");
		
		w = w.replace("", "o");
		
		w = w.replace("", "c");

		w = w.replace("", "s");
		w = w.replace("", "z");

		w = w.replace("", "ae");
		w = w.replace("", "oe");
		w = w.replace("", "ss");
		w = w.replace("", "th");
		w = w.replace("", "th");

		return w;
	}

	public static String removeEnclosures(String word) {
		String proc = word;
		for (String enclosure : ENCLOSURES) {
			proc = proc.replace(enclosure, "");
		}
		return proc;
	}

	public static String removeGarbage(String word) {
		String proc = word;
		for (String garbage : GARBAGE) {
			proc = proc.replace(garbage, "");
		}
		return proc;		
	}

//	public static List<String> tagWords(List<String> words) {
//		try {
//			
//			MaxentTagger tagger = new MaxentTagger("lib/models/left3words-wsj-0-18.tagger");
//			List<String> tags = new ArrayList<String>();
//			for (String word : words) {
//				tags.add(tagger.tagString(word));
//			}
//			return tags;
//		} catch (Exception e) {
//			throw new RuntimeException(e);
//		}
//	}
	
}
