package ob.nlp.wordNet.jwnl;

import java.util.ArrayList;
import java.util.Collection;

import ob.nlp.POS;
import ob.nlp.wordNet.Synset;

import net.didion.jwnl.JWNLException;
import net.didion.jwnl.data.Pointer;
import net.didion.jwnl.data.PointerType;
import net.didion.jwnl.data.Word;


public class JwnlSynset implements Synset {

	private net.didion.jwnl.data.Synset _synset;
	
	public JwnlSynset(net.didion.jwnl.data.Synset synset) {
		_synset = synset;
	}

	@Override
	public String toString() {
		return _synset.toString();
	}

	@Override
	public Collection<String> getSynonyms() {
		Word[] words = _synset.getWords();
		
//		Collection<Word> wordsByUsageCount = orderByUsageCount(words);
		
		Collection<String> synonyms = new ArrayList<String>(words.length);
		for (Word word : words) {
			synonyms.add(word.getLemma());
		}
		
		return synonyms;
	}

	@Override
	public Collection<Synset> getHypernyms() {
		return getRelatedSynsets(PointerType.HYPERNYM);
	}

	@Override
	public Collection<Synset> getHyponyms() {
		return getRelatedSynsets(PointerType.HYPONYM);
	}

	private Collection<Synset> getRelatedSynsets(PointerType type) {
		Collection<Synset> synsets = new ArrayList<Synset>();
		
		for (Pointer pointer : _synset.getPointers(type)) {
			try {
				synsets.add(new JwnlSynset(pointer.getTargetSynset()));
			} catch (JWNLException e) {
				throw new RuntimeException(e);
			}
		}

		return synsets;
	}

	public POS getPos() {
		return JwnlUtils.fromJwnlPos(_synset.getPOS());
	}

//	private Collection<Word> orderByUsageCount(Word[] words) {
//		Collection<Word> wordsByUsageCount = new TreeSet<Word>(new Comparator<Word>() {
//			public int compare(Word arg0, Word arg1) {
//				return arg0.getUsageCount() - arg1.getUsageCount();
//			}});
//		
//		for (Word word : words) {
//			wordsByUsageCount.add(word);
//		}
//		
//		return wordsByUsageCount;
//	}
}
