package ob.nlp.wordNet.jwnl;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collection;

import ob.nlp.POS;
import ob.nlp.wordNet.Synset;
import ob.nlp.wordNet.WordNet;


import net.didion.jwnl.JWNL;
import net.didion.jwnl.JWNLException;
import net.didion.jwnl.data.IndexWord;
import net.didion.jwnl.data.IndexWordSet;
import net.didion.jwnl.dictionary.Dictionary;

public class JwnlWordNet implements WordNet {

	private static JwnlWordNet _instance;
	
	private String _propertiesFile = "config/file_properties.xml";
	private Dictionary _dictionary;
	
	private JwnlWordNet() {
		try {
			JWNL.initialize(new FileInputStream(_propertiesFile));
		} catch (FileNotFoundException e) {
			throw new RuntimeException("file not found: " + _propertiesFile);
		} catch (JWNLException e) {
			throw new RuntimeException(e);
		}
		
		_dictionary = Dictionary.getInstance();
	}
	
	public static JwnlWordNet getInstance() {
		if (_instance == null) _instance = new JwnlWordNet();
		return _instance;
	}

	@Override
	public Collection<Synset> getSynsets(String lemma, POS pos) {
		return getSynsets(getWords(lemma, pos));
	}

	@Override
	public Collection<Synset> getSynsets(String lemma) {
		return getSynsets(lemma, null);
	}
	
	private Collection<Synset> getSynsets(IndexWordSet indexWordSet) {
		Collection<Synset> synsets = new ArrayList<Synset>();
		
		for (IndexWord word : indexWordSet.getIndexWordArray()) {
			for (net.didion.jwnl.data.Synset synset : getSenses(word)) {
				synsets.add(new JwnlSynset(synset));
//				for (net.didion.jwnl.data.Word w : synset.getWords()) {
//					System.out.print(w.getLemma() + ", ");					
//				}
//				System.out.print(";; ");
			}
		}
//		System.out.println();

		return synsets;
	}

	private net.didion.jwnl.data.Synset[] getSenses(IndexWord word) {
		try {
			return word.getSenses();
		} catch (JWNLException e) {
			throw new RuntimeException(e);
		}
	}

	private IndexWordSet getWords(String lemma, POS pos) {
		try {
			if (pos == null) return _dictionary.lookupAllIndexWords(lemma);
//			{
//				IndexWordSet ws = _dictionary.lookupAllIndexWords(lemma);
//				System.out.print(lemma + ": ");
//				for (IndexWord wi : ws.getIndexWordArray()) {
//					System.out.print(wi.getLemma() + ", ");
//				}
//				System.out.println();
//				return ws;
//			}
			else {
				IndexWordSet set = new IndexWordSet(lemma);
				set.add(_dictionary.lookupIndexWord(JwnlUtils.toJwnlPos(pos), lemma));
				return set;
			}
		} catch (JWNLException e) {
			throw new RuntimeException(e);
		}
	}

}
