package ob.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URL;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Or Biran
 * 
 * General utility methods.
 */
public class Utils {


	public static String ucFirst(String string) {
//		String lc = string.toLowerCase();
//		return lc.substring(0, 1).toUpperCase() + lc.substring(1);
		return string.substring(0, 1).toUpperCase() + string.substring(1);
	}
	
	public static Matcher getMatcher(String string, String regex) {
		Pattern pattern = Pattern.compile(regex);
		return pattern.matcher(string);
	}

	public static String matchGroup1(String string, String regex) {
		return getMatcher(string, regex).group(1);
	}

	public static String replaceSubstring(String string, int start, int end, String replacement) {
		return (start > 0 ? string.substring(0, start) : "") +
				replacement + 
				(end <= string.length() ? string.substring(end, string.length()) : "");
	}

	public static int countMatches(String string, String regex) {
		Matcher matcher = getMatcher(string, regex);
		int count = 0;
		while(matcher.find()) count++;
		return count;
	}

	public static int countOccurances(String string, String substring) {
		int count = 0;
		for (int index = string.indexOf(substring); index >= 0; index = string.indexOf(substring)) {
			count++;
			string = string.substring(index + 1);
		}
		return count;
	}
	
	public static String repeatString(String string, int times) {
		String repeat = "";
		for (int i=0; i<times; i++) repeat += string;
		return repeat;
	}
	
	public static <T> List<T> reverse(List<T> list) {
		List<T> reverseList = new ArrayList<T>();
		for (T object : list) reverseList.add(0, object);
		return reverseList;
	}

	public static <T> List<T> copy(List<T> list) {
		List<T> copy = new ArrayList<T>(list.size());
		copy.addAll(list);
//		Collections.copy(copy, list);
		return copy;
	}

	public static <T> List<T> list(T...objs) {
		List<T> list = new ArrayList<T>();
		for (T obj : objs) list.add(obj);
		return list;
	}
	
	public static int sumArray(int[] array) {
		int sum = 0;
		for (int i=0; i<array.length; i++) sum += array[i];
		return sum;
	}

	public static int sumCollection(Collection<Integer> collection) {
		int sum = 0;
		for (Integer integer : collection) sum += integer;
		return sum;
	}

	public static double doublePrecision(double x, int precision) {
		int precInt = (int)Math.pow(10, precision);
		long y = (long)(x * precInt);
		return (double)y / precInt;
	}

	private static FileReader getFileReader(String filename) {
		try {
			return new FileReader(filename);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("File not found: " + filename, e);
		}		
	}

	public static List<String> readLines(String filename) {
		BufferedReader reader = new BufferedReader(getFileReader(filename));
		List<String> lines = new ArrayList<String>();
		try {
			while (reader.ready()) {
				lines.add(reader.readLine());
			}
		} catch (IOException e) {
			throw new RuntimeException("Cannot read from file " + filename, e);
		}
		return lines;
	}

	public static String readText(String filename) {
		String text = "";
		for (String line : readLines(filename)) text += line + "\n";
		return text;
	}

	public static boolean fileExists(String filename) {
		File file = new File(filename);
		return file.exists();
	}

	public static <T> T coalesce(T value, T defaultValue) {
		return value == null ? defaultValue : value;
	}

	public static List<String> listFileNames(String dir) {
		File dirFile = new File(dir);
		String[] files = dirFile.list();
		return Arrays.asList(files);
	}

	public static List<File> listFiles(String dir) {
		File dirFile = new File(dir);
		File[] files = dirFile.listFiles();
		return Arrays.asList(files);
	}

	public static void writeToFile(String filename, String text) {
		try {
			PrintWriter writer = new PrintWriter(filename);
			writer.print(text);
			writer.close();
		} catch (IOException e) {
			throw new RuntimeException("cannot open file " + filename + " for writing", e);
		}
	}


	public static void writeLinesToFile(String filename, List<String> lines) {
		try {
			PrintWriter writer = new PrintWriter(filename);
			for (String line : lines) {
				writer.print(line + "\n");
			}
			writer.close();
		} catch (IOException e) {
			throw new RuntimeException("cannot open file " + filename + " for writing", e);
		}
	}

	public static boolean containsIgnoreCase(String text, String target) {
		return text.toLowerCase().contains(target.toLowerCase());
	}

	public static boolean containsIgnoreCase(Collection<String> strings, String target) {
		for (String string : strings) {
			if (string.toLowerCase().equals(target.toLowerCase())) return true;
		}
		return false;
	}

	public static Date parseDate(String string, DateFormat format) {
		try {
			return format.parse(string);
		} catch (ParseException e) {
			throw new RuntimeException(e);
		}
	}

	public static String formatDouble(Double d) {
		return NumberFormat.getInstance().format(d);
	}

	public static List<String> linesFromUrl(String url) {
		try {
			URL u = new URL(url);
			BufferedReader reader = new BufferedReader(new InputStreamReader(u.openStream()));

			List<String> lines = new ArrayList<String>();
			while (reader.ready()) {
				lines.add(reader.readLine());
			}

			reader.close();
			
			return lines;
			
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	public static String textFromUrl(String url) {
		try {
			URL u = new URL(url);
			BufferedReader reader = new BufferedReader(new InputStreamReader(u.openStream()));

			String text = "";
			while (reader.ready()) {
				text += reader.readLine();
			}

			reader.close();
			
			return text;
			
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
